%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Matlab example of using SPISimProxy to do AMI_Init            www.SPISim.com
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc

%% file to load data from
inpFile = 'SPISimProxy_Init_Inp.txt';
%% file to save data to
outFile = 'SPISimProxy_Init_Out.txt';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Parse data generated by SPISimProxy for AMI_Init processing
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% open file handle
fid=fopen(inpFile, 'rt');
tmp = textscan(fid,'%s','Delimiter','\n');
fclose(fid);

tmp = tmp{1};   
stage = 0;
numLine = size(tmp, 1);

% parse line by line
for i = 2:numLine
  line = tmp{i};

  % line after comment line is the data
  if (strncmpi(line, '#', 1))
    stage = stage + 1;
    myDat = tmp{i + 1};
    switch(stage)
    case 1
      k = 1;
      for j=i+1:numLine
        line = tmp{j};
        if (strncmpi(line, '#', 1) == 0)
          wave(k) = str2double(cellstr(tmp{j}));
          k = k + 1;
        end
      end
      i = j - 1;
    case 2
      num_of_rows = str2double(myDat);
      wave = wave(1:num_of_rows);
    case 3
      aggressors = str2double(myDat);
 
    case 4
      sample_interval = str2double(myDat);
 
    case 5
      bit_time = str2double(myDat);
 
    case 6
      parm_in = myDat;
 
    case 7
      parm_out = myDat;
 
    case 8
      msg = myDat;
 
    end
    i = i + 1;
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Perform AMI_Init using Matlab
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sample_per_bit = floor(bit_time / sample_interval);
preTap  = -0.05;
posTap  = -0.1;
mainSig = 1 - abs(preTap) - abs(posTap);
waveInp = wave;
ht = [[1 zeros(1, sample_per_bit-1)] * preTap ...
      [1 zeros(1, sample_per_bit-1)] * mainSig ...
      [1 zeros(1, sample_per_bit-1)] * posTap];
out=conv(wave, ht);
wave = out(1:size(wave, 2));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Store data for SPISimProxy to parse and return to the simulator
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% open file handle
fid=fopen(outFile, 'w');
 
% start writing field by field
fprintf(fid, '### GENERATED BY MATLAB BY SPISIM ###\n');
 
fprintf(fid, '# wave\n');
for i=1:num_of_rows
  fprintf(fid, '%g\n', wave(i));
end
 
fprintf(fid, '# number_of_rows\n');
fprintf(fid, '%d\n', num_of_rows);
 
fprintf(fid, '# aggressors\n');
fprintf(fid, '%d\n', aggressors);
 
fprintf(fid, '# sample_interval\n');
fprintf(fid, '%g\n', sample_interval);
 
fprintf(fid, '# bit_time\n');
fprintf(fid, '%g\n', bit_time);
 
fprintf(fid, '# AMI_parameters_in\n');
fprintf(fid, '%s\n', parm_in);
 
fprintf(fid, '# AMI_parameters_out\n');
fprintf(fid, '%s\n', parm_out);
 
fprintf(fid, '# msg\n');
fprintf(fid, '%s\n', msg);
 
fclose(fid);
